/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_OPTIONAL_SOME_VALUE_HPP
#define ASYNCHRONOUS_SIGNALS_OPTIONAL_SOME_VALUE_HPP 1

#include <boost/optional.hpp>

namespace asynchronous
{

  namespace signals
  {

    template < typename Signature >
    class optional_some_value;

    template < typename Result, typename ... Args >
    class optional_some_value< Result(Args...) >
    {
      using self = optional_some_value;
      using optional_value = boost::optional<Result>;

    public:
      void merge (self const& other)
      {
	if (!value_)
	  value_ = other.value_;
      }	

      void operator () (Result result) { value_ = result; }

      auto value () const { return value_; }

    private:
      optional_value value_;
    };

  } // namespace signals

} // namespace asynchronous

#endif
