/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_SIGNAL_DIAGNOSTICS_HPP
#define ASYNCHRONOUS_SIGNALS_SIGNAL_DIAGNOSTICS_HPP 1

#include <iosfwd>
#include <list>
#include <map>
#include <string>
#include <boost/asynchronous/diagnostics/diagnostic_item.hpp>
#include <boost/chrono/chrono_io.hpp>
#include <boost/filesystem/path.hpp>

namespace asynchronous
{

  namespace signals
  {

    class signal_diagnostics
    {
      using self = signal_diagnostics;

      using diagnostic_item = boost::asynchronous::diagnostic_item;
      using diagnostic_list = std::list<diagnostic_item>;
      using diagnostic_map = std::map<std::string,diagnostic_list>;

      using path_type = boost::filesystem::path;

    public:
      static auto header ()
      { return "name\ttype\tposted\tstarted\tfinished\twaited\tlasted"; }

      signal_diagnostics (std::string name, diagnostic_map map)
	: name_(name)
	, map_(map)
      {}

      friend std::ostream& operator << (std::ostream& os, self const& diagnostics)
      {
	constexpr char tab = '\t';

	for (auto& diagnostic : diagnostics.map_)
	  {
	    auto type = diagnostic.first;
	    for (auto& item : diagnostic.second)
	      os << diagnostics.name_ << tab
		 << type << tab
		 << boost::chrono::nanoseconds(item.get_posted_time().time_since_epoch()).count() / 1000 << tab
		 << boost::chrono::nanoseconds(item.get_started_time().time_since_epoch()).count() / 1000 << tab
		 << boost::chrono::nanoseconds(item.get_finished_time().time_since_epoch()).count() / 1000 << tab
		 << boost::chrono::nanoseconds(item.get_started_time() - item.get_posted_time()).count() / 1000 << tab
		 << boost::chrono::nanoseconds(item.get_finished_time() - item.get_started_time()).count() / 1000
		 << std::endl;
	  }
	return os;
      }

    private:
      std::string name_;
      diagnostic_map map_;
    };

  } // namespace signals

} // namespace asynchronous

#endif
