/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#ifndef GRAPH_MODEL_ACCUMULATORS_ACCUMULATOR_SET_HPP
#define GRAPH_MODEL_ACCUMULATORS_ACCUMULATOR_SET_HPP 1

#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/count.hpp>
#include <boost/accumulators/statistics/mean.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/variance.hpp>

namespace graph_model
{

  namespace accumulators
  {

    template < typename Variate >
    class accumulator_set
    {
      using count = boost::accumulators::tag::count;
      using mean = boost::accumulators::tag::mean;
      using variance = boost::accumulators::tag::variance;

      using stats = boost::accumulators::stats<count,mean,variance>;

      using accumulator_set_type = boost::accumulators::accumulator_set<Variate,stats>;

    public:
      using type = accumulator_set_type;
    };

    template < typename Variate >
    using accumulator_set_t = typename accumulator_set<Variate>::type;

    template < typename Variate >
    auto make_accumulator_set ()
    {
      using accumulator_set_type = accumulator_set_t<Variate>;
      return accumulator_set_type();
    }

  } // namespace accumulators

} // namespace graph_model

#endif
