/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_DISTRIBUTION_DISTRIBUTION_POLICY_HPP
#define GRAPH_MODEL_DISTRIBUTION_DISTRIBUTION_POLICY_HPP 1

#include <graph_model/random/random_number_generator.hpp>

namespace graph_model
{

  namespace distribution
  {

    template < typename DistributionPolicy
	       , typename GeneratorPolicy
	       >
    class distribution_policy
    {
      using self = distribution_policy;

      template < typename ... Parameters >
      static auto& generator () { return GeneratorPolicy::template apply<Parameters...>(); }

      template < typename ... Parameters >
      static auto params (Parameters ... parameters)
      {
	using param_type
	  = typename std::decay_t<decltype(generator<Parameters...>())>::param_type;
	return param_type(parameters...);
      }

      static auto& random_number_generator ()
      { return random::random_number_generator::apply(); }

    protected:
      template < typename Variate, typename ... Parameters >
      static auto distribution (Parameters ... parameters)
      { return DistributionPolicy::template apply<Variate>(parameters...); }

    public:
      template < typename ... Parameters >
      static auto sample (Parameters ... parameters)
      {
	return generator<Parameters...>()(random_number_generator(),params(parameters...));
      }

      template < typename Variate, typename ... Parameters >
      static auto eval_pdf (Variate variate, Parameters ... parameters)
      {
	auto distribution = self::distribution<Variate>(parameters...);
	return pdf(distribution,variate);
      }
    };

  } // namespace distribution

} // namespace graph_model

#endif
