/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_EDGE_FACTOR_EDGE_DESCRIPTOR_HPP
#define GRAPH_MODEL_EDGE_FACTOR_EDGE_DESCRIPTOR_HPP 1

#include <iosfwd>
#include <graph_model/vertex/vertex_descriptor.hpp>

namespace graph_model
{

  namespace edge
  {

    template < typename VertexDescriptor >
    class edge_descriptor
      : boost::totally_ordered<edge_descriptor<VertexDescriptor>>
    {
      using self = edge_descriptor;

    public:
      edge_descriptor () = default;
      edge_descriptor (VertexDescriptor source, VertexDescriptor target)
	: source_(source)
	, target_(target)
      {}

      auto source () const { return source_; }
      auto target () const { return target_; }

      friend bool operator == (self left, self right)
      { return left.source_ == right.source_ && left.target_ == right.target_; }

      friend bool operator < (self left, self right)
      {
	return (left.source_ < right.source_)
	  || (left.source_ == right.source_ && left.target_ < right.target_);
      }

      friend std::ostream& operator << (std::ostream& os, self u)
      { return os << u.source_ << " -> " << u.target_; }

    private:
      VertexDescriptor source_;
      VertexDescriptor target_;
    };

    template < typename VertexDescriptor >
    auto make_edge_descriptor (VertexDescriptor source, VertexDescriptor target)
    { return edge_descriptor<VertexDescriptor>(source,target); }

  } // namespace edge

} // namespace graph_model

#endif
