/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_RANDOM_UNIFORM_01_HPP
#define GRAPH_MODEL_RANDOM_UNIFORM_01_HPP 1

#include <boost/random/uniform_01.hpp>
#include <graph_model/random/random_number_generator.hpp>

namespace graph_model
{

  namespace random
  {

    template < typename Real = double >
    class uniform_01
    {
      using distribution_type = boost::random::uniform_01<Real>;

    public:
      static auto apply () { return apply(rng()); }

      template < typename Engine >
      static auto apply (Engine& engine) { return distribution()(engine); }

      static void reset  () { distribution().reset(); }

      auto operator () () { return apply(); }

      template < typename Engine >
      auto operator () (Engine& engine) { return apply(engine); }

    private:
      static distribution_type& distribution ()
      {
	static distribution_type distribution;
	return distribution;
      }

      static auto& rng () { return random_number_generator::apply(); }
    };

  } // namespace random

} // namespace graph_model

#endif
