/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_DISTRIBUTION_HPP
#define GRAPH_MODEL_VERTEX_DISTRIBUTION_HPP 1

#include <graph_model/vertex/distribution_fwd.hpp>
#include <graph_model/vertex/distribution_base.hpp>
#include <graph_model/vertex/distribution_impl_base.hpp>
#include <graph_model/vertex/expression_terminal.hpp>
#include <graph_model/vertex/strategy/basic_distribution.hpp>
#include <graph_model/vertex/strategy_owner_base.hpp>
#include <graph_model/vertex/tags.hpp>
#include <graph_model/vertex/vertex_expression.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Variate, typename Probability >
    class distribution
      : public distribution_impl_base< distribution<Variate,Probability>
				       , tag::distribution
				       , distribution_base<Variate,Probability>
				       >
      , public strategy_owner_base< distribution<Variate,Probability>
				    , strategy::basic_distribution_factory<Variate,Probability>
				    >
    {};

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<distribution<T...>> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<distribution<T...>&> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename Variate, typename Probability, typename ... Args >
    auto make_distribution (Args&& ... args)
    {
      using distribution_type = distribution<Variate,Probability>;
      distribution_type distribution(std::forward<Args>(args)...);
      return distribution;
    }

  } // namespace vertex

} // namespace graph_model

#endif
