/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_PDF_RV_IMPL_BASE_HPP
#define GRAPH_MODEL_VERTEX_PDF_RV_IMPL_BASE_HPP 1

namespace graph_model
{

  namespace vertex
  {

    /// \brief Delegate \c pdf() to an observed distribution
    ///
    template < typename Derived, typename Base >
    class pdf_rv_impl_base
      : public Base
    {
      using base = Base;

    public:
      auto pdf () const { return distribution_().pdf(); }

    private:
      typename base::probability_type pdf_ () const override { return pdf(); }

    private:
      auto& distribution_ () { return derived().distribution(); }
      auto& distribution_ () const { return derived().distribution(); }

    private:
      auto& derived () { return static_cast<Derived&>(*this); }
      auto& derived () const { return static_cast<Derived const&>(*this); }
    };

  } // namespace vertex

} // namespace graph_model

#endif
