/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_TYPE_VERTEX_CATEGORY_BASE_HPP
#define GRAPH_MODEL_VERTEX_TYPE_VERTEX_CATEGORY_BASE_HPP 1

#include <boost/variant/variant.hpp>
#include <graph_model/vertex/tags.hpp>
#include <use_default.hpp>

namespace graph_model
{

  namespace vertex
  {

    namespace type
    {

      template < typename VertexCategory = use_default<> >
      struct vertex_category_base
      {
	using vertex_category_type = VertexCategory;
      };

      template <>
      struct vertex_category_base< use_default<> >
      {
	using vertex_category_type
	= boost::variant< tag::vertex
			  , tag::expression
			  , tag::factor
			  , tag::scalar_factor
			  , tag::distribution
			  , tag::scalar_distribution
			  , tag::random_variable
			  , tag::scalar_random_variable
			  , tag::summarized_random_variable
			  , tag::summarized_scalar_random_variable
			  >;
      };

    } // namespace type

  } // namespace vertex

} // namespace graph_model

#endif
