/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_VECTOR_SIMPLEX_FWD_HPP
#define SAMPLE_SPACE_VECTOR_SIMPLEX_FWD_HPP 1

#include <limits>
#include <boost/accumulators/accumulators.hpp>
#include <sample_space/vector/tag/simplex.hpp>
#include <traits/convert.hpp>
#include <traits/domain.hpp>
#include <traits/key_type.hpp>
#include <traits/mapped_type.hpp>
#include <traits/probability.hpp>
#include <traits/real.hpp>
#include <traits/value.hpp>
#include <traits/variate.hpp>
#include <type_traits/is_map_fwd.hpp>

namespace sample_space
{

  namespace vector
  {

    /// \brief A vector on a simplex
    /// \tparam T The value type
    ///
    template < typename T
	       , typename Index = void
	       >
    class simplex;

  } // namespace vector
  
} // namespace sample_space

namespace std
{

  template < typename T, typename Index >
  struct numeric_limits< sample_space::vector::simplex<T,Index> >
    : numeric_limits<T>
  {};
  
} // namespace std

namespace traits
{

  namespace defaults
  {

    template < typename T, typename Index, typename U >
    struct convert< sample_space::vector::simplex<T,Index>,U >
    {
      using type = sample_space::vector::simplex<U,Index>;
    };
  
    template < typename T, typename Index, typename U >
    struct convert< sample_space::vector::simplex<T,Index> const,U >
    {
      using type = sample_space::vector::simplex<U,Index> const;
    };

    template < typename T, typename Index >
    struct domain< sample_space::vector::simplex<T,Index>,std::false_type >
      : domain<T>
    {};

    template < typename T, typename Index >
    struct probability< sample_space::vector::simplex<T,Index> >
      : probability<T>
    {};

    template < typename T, typename Index >
    struct real< sample_space::vector::simplex<T,Index>,std::false_type,std::false_type >
      : real<T>
    {};

    template < typename T, typename Index >
    struct value< sample_space::vector::simplex<T,Index> >
    {
      using type = T;
    };

    template < typename T, typename Index >
    struct variate< sample_space::vector::simplex<T,Index> >
    {
      using type = sample_space::vector::simplex<T,Index>;
    };

  } // namespace defaults

  template < typename T >
  class key_type< sample_space::vector::simplex<T,void> >
  {};

  template < typename T, typename Index >
  struct key_type< sample_space::vector::simplex<T,Index> >
  { using type = Index; };

  template < typename T >
  class mapped_type< sample_space::vector::simplex<T,void> >
  {};

  template < typename T, typename Index >
  struct mapped_type< sample_space::vector::simplex<T,Index> >
  { using type = T; };

} // namespace traits

namespace type_traits
{

  template < typename T >
  struct is_map< sample_space::vector::simplex<T,void> >
    : std::false_type
  {};

  template < typename T, typename Index >
  struct is_map< sample_space::vector::simplex<T,Index> >
    : std::true_type
  {};

} // namespace type_traits

namespace boost
{

  namespace numeric
  {

    namespace functional
    {

      template < typename T, typename Index >
      struct tag< sample_space::vector::simplex<T,Index> >
      {
	using type = sample_space::vector::tag::simplex;
      };

    } // namespace functional

  } // namespace numeric

} // namespace boost

#endif
