/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef TYPE_TRAITS_COMMON_TYPE_HPP
#define TYPE_TRAITS_COMMON_TYPE_HPP 1

#include <type_traits>

namespace type_traits
{

  template < typename ... T >
  class common_type;

  namespace defaults
  {

    template < typename ... T >
    class common_type;

    template < typename T >
    struct common_type< T >
    {
      using type = T;
    };

    template < typename T, typename U >
    struct common_type< T,U >
    {
      using type = std::common_type_t<T,U>;
    };

    template < typename T, typename U, typename ... V >
    struct common_type< T,U,V... >
      : common_type< typename type_traits::common_type<T,U>::type,V... >
    {};

  } // namespace defaults

  namespace customization
  {

    template < typename ... T >
    struct common_type
      : defaults::common_type<T...>
    {};

  } // namespace customization

  template < typename ... T >
  struct common_type
    : customization::common_type<T...>
  {};

  template < typename ... T >
  using common_type_t = typename common_type<T...>::type;
  
} // namespace type_traits

#endif
