/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE dataframe_manipulator_storage_base

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <dataframe/manipulator_storage_base.hpp>
#include <array>
#include <boost/mpl/vector.hpp>

template < typename T >
class manipulator
  : public dataframe::manipulator_storage_base< manipulator<T>,T >
{
  using base = dataframe::manipulator_storage_base< manipulator<T>,T >;
    
public:
  template < typename ... Args >
  manipulator (Args&& ... args)
    : base(std::forward<Args>(args)...)
  {}
};

template < typename T, typename ... Args >
auto make_manipulator (Args&& ... args) { return manipulator<T>(std::forward<Args>(args)...); }

struct big_class
{
  big_class () = default;
  explicit big_class (int i) : i_(i) {}
  operator int () const { return i_; }

private:
  int i_;
  std::array<char,1024> data_;
};

static_assert(sizeof(big_class) > sizeof(long),"");

using test_types = boost::mpl::vector<int,long,unsigned,long long,unsigned long long,float,double,long double,big_class>;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_make_manipulator,T,test_types)
{
  { auto manipulator = make_manipulator<T>(); }
  { auto manipulator = make_manipulator<T>(T()); }
  { auto manipulator = make_manipulator<T>(std::cout); }
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_value_initialized,T,test_types)
{
  auto manipulator = make_manipulator<T>(T(42));
  BOOST_CHECK_EQUAL(manipulator.value(),T(42));
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_stream_initialized,T,test_types)
{
  auto manipulator = make_manipulator<T>(std::cout);
  BOOST_CHECK_EQUAL(manipulator.value(),T(0));
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_streamed,T,test_types)
{
  std::cout << make_manipulator<T>(T(42));
  auto manipulator = make_manipulator<T>(std::cout);
  BOOST_CHECK_EQUAL(manipulator.value(),T(42));
}
