/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_vertex_summarized_random_variable

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <asynchronous/signals/type/default_signal_executor_is_sequential.hpp>
#include <graph_model/vertex/summarized_random_variable.hpp>

#include <boost/mpl/pair.hpp>
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector
  <
  boost::mpl::pair<int,double>
  , boost::mpl::pair<long,double>
  , boost::mpl::pair<float,double>
  , boost::mpl::pair<double,double>
  , boost::mpl::pair<long double,double>
  >;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_default_constructor,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using summarized_random_variable_type = graph_model::vertex::summarized_random_variable<variate_type,probability_type>;
  summarized_random_variable_type rv;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_constructor_function,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using graph_model::vertex::make_summarized_random_variable;
  auto rv = make_summarized_random_variable<variate_type,probability_type>();
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_accumulator,Pair,test_types)
{
  using variate_type = typename Pair::first;
  using probability_type = typename Pair::second;
  using boost::accumulators::count;
  using boost::accumulators::mean;
  using boost::accumulators::variance;
  using graph_model::vertex::make_summarized_random_variable;
  auto rv = make_summarized_random_variable<variate_type,probability_type>();
  BOOST_CHECK_EQUAL(count(rv.accumulator()),0);
  rv.reset_proposed_variate(1);
  BOOST_CHECK_EQUAL(rv.proposed_variate(),1);
  rv.accept();
  rv.commit();
  BOOST_CHECK_EQUAL(rv.current_variate(),1);
  BOOST_CHECK_EQUAL(count(rv.accumulator()),1);
  BOOST_CHECK_EQUAL(mean(rv.accumulator()),1);
  BOOST_CHECK_EQUAL(variance(rv.accumulator()),0);
}
